[−][src]Enum keyed_priority_queue::Entry
A view into a single entry in a queue, which may either be vacant or occupied.
This enum is constructed from the entry method on KeyedPriorityQueue.
Variants
Occupied(OccupiedEntry<'a, TKey, TPriority, S>)An occupied entry.
Vacant(VacantEntry<'a, TKey, TPriority, S>)A vacant entry.